@@ -10,7 +10,7 @@
 import flexsolve as flx
 from numba import njit
 from warnings import warn
-from ..exceptions import InfeasibleRegion
+from ..exceptions import InfeasibleRegion, NoEquilibrium
 from . import binary_phase_fraction as binary
 from .equilibrium import Equilibrium
 from .dew_point import DewPointCache
@@ -276,9 +276,18 @@ class VLE(Equilibrium, phases='lg'):
         # Run equilibrium
         if T_spec:
             if P_spec:
-                self.set_thermal_condition(T, P)
+                try:
+                    self.set_thermal_condition(T, P)
+                except NoEquilibrium:
+                    thermal_condition = self._thermal_condition
+                    thermal_condition.T = T
+                    thermal_condition.P = P
             elif V_spec:
-                self.set_TV(T, V)
+                try:
+                    self.set_TV(T, V)
+                except NoEquilibrium:
+                    thermal_condition = self._thermal_condition
+                    thermal_condition.T = T
             elif H_spec:
                 self.set_TH(T, H)
             elif x_spec:
@@ -287,9 +296,18 @@ class VLE(Equilibrium, phases='lg'):
                 self.set_Ty(T, np.asarray(y))
         elif P_spec:
             if V_spec:
-                self.set_PV(P, V)
+                try:
+                    self.set_PV(P, V)
+                except NoEquilibrium:
+                    thermal_condition = self._thermal_condition
+                    thermal_condition.P = P
             elif H_spec:
-                self.set_PH(P, H)
+                try:
+                    self.set_PH(P, H)
+                except NoEquilibrium:
+                    thermal_condition = self._thermal_condition
+                    thermal_condition.P = P
+                    thermal_condition.T = self.mixture.xH(self._phase_data, T, P)
             elif x_spec:
                 self.set_Px(P, np.asarray(x))
             else: # y_spec
@@ -329,7 +347,7 @@ class VLE(Equilibrium, phases='lg'):
             self._index = index
         
         # Get overall composition
-        if not mol.any(): raise RuntimeError('no chemicals to perform equilibrium')
+        if not mol.any(): raise NoEquilibrium('no chemicals to perform equilibrium')
         self._F_mass = (chemicals.MW * mol).sum()
         self._mol_vle = mol_vle = mol[index]
 
@@ -344,6 +362,7 @@ class VLE(Equilibrium, phases='lg'):
         self._F_mol_heavy = F_mol_heavy = (heavy_mol * chemicals._heavy_solutes).sum()
         self._F_mol_vle = F_mol_vle = mol_vle.sum()
         self._F_mol = F_mol = F_mol_vle + F_mol_light + F_mol_heavy
+        if F_mol == 0.: raise NoEquilibrium('no chemicals to perform equilibrium')
         self._z = mol_vle / F_mol
         self._z_light = z_light = F_mol_light / F_mol
         self._z_heavy = z_heavy = F_mol_heavy / F_mol
@@ -719,7 +738,7 @@ class VLE(Equilibrium, phases='lg'):
         
         # Check if super heated vapor
         T_dew, x_dew = self._dew_point.solve_Tx(self._z, P)
-        if self._F_mol_heavy: T_dew = self._dew_point.Tmax
+        if self._F_mol_heavy or T_dew < T_bubble: T_dew = self._dew_point.Tmax
         vapor_mol[index] = mol
         liquid_mol[index] = 0
         H_dew = self.mixture.xH(self._phase_data, T_dew, P)
@@ -756,7 +775,9 @@ class VLE(Equilibrium, phases='lg'):
                                      H_hat_bubble - H_hat, H_hat_dew - H_hat,
                                      self._T, self.T_tol, self.H_hat_tol,
                                      (H_hat,), checkiter=False, checkbounds=False)
-            # self._T = thermal_condition.T = T
+            
+            
+            
             # Make sure enthalpy balance is correct
             self._T = thermal_condition.T = self.mixture.xsolve_T(
                 self._phase_data, H, T, P
@@ -771,8 +792,11 @@ class VLE(Equilibrium, phases='lg'):
         self._V = V
         self._y = fn.normalize(v, v.sum() + self._F_mol_light)
         z_last = self._z_last
-        if (self._x is None
-            or np.abs(z_last - self._z).sum() > 0.001):
+        try:
+            reload_cache = self._x is None or np.abs(z_last - self._z).sum() > 0.001
+        except:
+            reload_cache = True
+        if reload_cache:
             l = self._mol_vle - v
             self._x = fn.normalize(l, l.sum() + self._F_mol_heavy)
     
@@ -808,7 +832,7 @@ class VLE(Equilibrium, phases='lg'):
         xV[:-1] = x
         xV[-1] = self._V
         xV = flx.aitken(f, xV, 1e-12, args, checkiter=False, 
-                       checkconvergence=False, convergenceiter=3)
+                       checkconvergence=False, convergenceiter=5)
         x = xV[:-1]
         self._V = V = xV[-1]
         x[x < 1e-32] = 1e-32
@@ -833,7 +857,7 @@ class VLE(Equilibrium, phases='lg'):
                            args=(Psats_over_P, T, P),
                            checkiter=False, 
                            checkconvergence=False, 
-                           convergenceiter=3)
+                           convergenceiter=5)
         self._v = v = self._F_mol * self._V * y
         mask = v > self._mol_vle
         v[mask] = self._mol_vle[mask]
